@@ -219,8 +219,8 @@ namespace KeePass.Forms
 
 			bool bHideInitial = m_cbHidePassword.Checked;
 			m_icgPassword.Attach(m_tbPassword, m_cbHidePassword, m_lblPasswordRepeat,
-				m_tbRepeatPassword, m_lblQuality, m_pbQuality, m_lblQualityBitsText,
-				this, bHideInitial, false);
+				m_tbRepeatPassword, m_lblQuality, m_pbQuality, m_lblQualityInfo,
+				m_ttRect, this, bHideInitial, false);
 
 			if(m_pwEntry.Expires)
 			{
@@ -291,6 +291,12 @@ namespace KeePass.Forms
 
 		// Public for plugins
 		public void UpdateEntryStrings(bool bGuiToInternal, bool bSetRepeatPw)
+		{
+			UpdateEntryStrings(bGuiToInternal, bSetRepeatPw, false);
+		}
+
+		public void UpdateEntryStrings(bool bGuiToInternal, bool bSetRepeatPw,
+			bool bUpdateState)
 		{
 			if(bGuiToInternal)
 			{
@@ -344,6 +350,8 @@ namespace KeePass.Forms
 				}
 				UIUtil.SetTopVisibleItem(m_lvStrings, iTopVisible);
 			}
+
+			if(bUpdateState) EnableControlsEx();
 		}
 
 		// Public for plugins
@@ -461,7 +469,7 @@ namespace KeePass.Forms
 			m_lvAutoType.Columns.Add(KPRes.TargetWindow, nWidth);
 			m_lvAutoType.Columns.Add(KPRes.Sequence, nWidth);
 
-			UpdateAutoTypeList();
+			UpdateAutoTypeList(false);
 
 			if(m_pwEditMode == PwEditMode.ViewReadOnlyEntry)
 			{
@@ -475,7 +483,7 @@ namespace KeePass.Forms
 			}
 		}
 
-		private void UpdateAutoTypeList()
+		private void UpdateAutoTypeList(bool bUpdateState)
 		{
 			m_lvAutoType.Items.Clear();
 
@@ -485,6 +493,8 @@ namespace KeePass.Forms
 				ListViewItem lvi = m_lvAutoType.Items.Add(a.WindowName, (int)PwIcon.List);
 				lvi.SubItems.Add((a.Sequence.Length > 0) ? a.Sequence : strDefault);
 			}
+
+			if(bUpdateState) EnableControlsEx();
 		}
 
 		private void InitHistoryTab()
@@ -496,7 +506,7 @@ namespace KeePass.Forms
 			m_lvHistory.Columns.Add(KPRes.UserName);
 			m_lvHistory.Columns.Add(KPRes.Size, 72, HorizontalAlignment.Right);
 
-			UpdateHistoryList();
+			UpdateHistoryList(false);
 
 			if(m_pwEditMode == PwEditMode.ViewReadOnlyEntry)
 			{
@@ -505,7 +515,7 @@ namespace KeePass.Forms
 			}
 		}
 
-		private void UpdateHistoryList()
+		private void UpdateHistoryList(bool bUpdateState)
 		{
 			m_lvHistory.Items.Clear();
 
@@ -518,6 +528,8 @@ namespace KeePass.Forms
 				lvi.SubItems.Add(pe.Strings.ReadSafe(PwDefs.UserNameField));
 				lvi.SubItems.Add(StrUtil.FormatDataSizeKB(pe.GetSize()));
 			}
+
+			if(bUpdateState) EnableControlsEx();
 		}
 
 		private void ResizeColumnHeaders()
@@ -636,7 +648,7 @@ namespace KeePass.Forms
 			InitAutoTypeTab();
 			InitHistoryTab();
 
-			UpdateEntryStrings(false, true);
+			UpdateEntryStrings(false, true, false);
 			UpdateEntryBinaries(false, false);
 
 			if(PwDefs.IsTanEntry(m_pwEntry)) m_btnTools.Enabled = false;
@@ -771,7 +783,7 @@ namespace KeePass.Forms
 			peTarget.Expires = m_cgExpiry.Checked;
 			if(peTarget.Expires) peTarget.ExpiryTime = m_cgExpiry.Value;
 
-			UpdateEntryStrings(true, false);
+			UpdateEntryStrings(true, false, false);
 
 			peTarget.Strings = m_vStrings;
 			peTarget.Binaries = m_vBinaries;
@@ -886,14 +898,14 @@ namespace KeePass.Forms
 		{
 			if(m_pwEditMode == PwEditMode.ViewReadOnlyEntry) return;
 
-			UpdateEntryStrings(true, false);
+			UpdateEntryStrings(true, false, false);
 
 			EditStringForm esf = new EditStringForm();
 			esf.InitEx(m_vStrings, null, null, m_pwDatabase);
 
 			if(UIUtil.ShowDialogAndDestroy(esf) == DialogResult.OK)
 			{
-				UpdateEntryStrings(false, false);
+				UpdateEntryStrings(false, false, true);
 				ResizeColumnHeaders();
 			}
 		}
@@ -905,7 +917,7 @@ namespace KeePass.Forms
 			ListView.SelectedListViewItemCollection vSel = m_lvStrings.SelectedItems;
 			if(vSel.Count <= 0) return;
 
-			UpdateEntryStrings(true, false);
+			UpdateEntryStrings(true, false, false);
 
 			string strName = vSel[0].Text;
 			ProtectedString psValue = m_vStrings.Get(strName);
@@ -914,24 +926,24 @@ namespace KeePass.Forms
 			EditStringForm esf = new EditStringForm();
 			esf.InitEx(m_vStrings, strName, psValue, m_pwDatabase);
 			if(UIUtil.ShowDialogAndDestroy(esf) == DialogResult.OK)
-				UpdateEntryStrings(false, false);
+				UpdateEntryStrings(false, false, true);
 		}
 
 		private void OnBtnStrDelete(object sender, EventArgs e)
 		{
 			if(m_pwEditMode == PwEditMode.ViewReadOnlyEntry) return;
 
-			UpdateEntryStrings(true, false);
+			UpdateEntryStrings(true, false, false);
 
-			ListView.SelectedListViewItemCollection lvsicSel = m_lvStrings.SelectedItems;
-			for(int i = 0; i < lvsicSel.Count; ++i)
+			ListView.SelectedListViewItemCollection lvsc = m_lvStrings.SelectedItems;
+			foreach(ListViewItem lvi in lvsc)
 			{
-				if(!m_vStrings.Remove(lvsicSel[i].Text)) { Debug.Assert(false); }
+				if(!m_vStrings.Remove(lvi.Text)) { Debug.Assert(false); }
 			}
 
-			if(lvsicSel.Count > 0)
+			if(lvsc.Count > 0)
 			{
-				UpdateEntryStrings(false, false);
+				UpdateEntryStrings(false, false, true);
 				ResizeColumnHeaders();
 			}
 		}
@@ -948,14 +960,16 @@ namespace KeePass.Forms
 			UpdateEntryBinaries(true, false);
 
 			ListView.SelectedListViewItemCollection lvsc = m_lvBinaries.SelectedItems;
-			int nSelCount = lvsc.Count;
-			if(nSelCount == 0) { Debug.Assert(false); return; }
+			foreach(ListViewItem lvi in lvsc)
+			{
+				if(!m_vBinaries.Remove(lvi.Text)) { Debug.Assert(false); }
+			}
 
-			for(int i = 0; i < nSelCount; ++i)
-				m_vBinaries.Remove(lvsc[nSelCount - i - 1].Text);
-
-			UpdateEntryBinaries(false, true);
-			ResizeColumnHeaders();
+			if(lvsc.Count > 0)
+			{
+				UpdateEntryBinaries(false, true);
+				ResizeColumnHeaders();
+			}
 		}
 
 		private void OnBtnBinSave(object sender, EventArgs e)
@@ -1026,7 +1040,7 @@ namespace KeePass.Forms
 
 			if(UIUtil.ShowDialogAndDestroy(dlg) == DialogResult.OK)
 			{
-				UpdateAutoTypeList();
+				UpdateAutoTypeList(true);
 				ResizeColumnHeaders();
 			}
 		}
@@ -1043,7 +1057,7 @@ namespace KeePass.Forms
 				m_vStrings);
 
 			if(UIUtil.ShowDialogAndDestroy(dlg) == DialogResult.OK)
-				UpdateAutoTypeList();
+				UpdateAutoTypeList(true);
 		}
 
 		private void OnBtnAutoTypeDelete(object sender, EventArgs e)
@@ -1060,7 +1074,7 @@ namespace KeePass.Forms
 					m_atConfig.RemoveAt(j);
 			}
 
-			UpdateAutoTypeList();
+			UpdateAutoTypeList(true);
 			ResizeColumnHeaders();
 		}
 
@@ -1087,15 +1101,19 @@ namespace KeePass.Forms
 
 			Debug.Assert(m_vHistory.UCount == m_lvHistory.Items.Count);
 
-			ListView.SelectedIndexCollection lvsi = m_lvHistory.SelectedIndices;
-			int nSelCount = lvsi.Count;
+			ListView.SelectedIndexCollection lvsc = m_lvHistory.SelectedIndices;
+			int n = lvsc.Count; // Getting Count sends a message
+			if(n == 0) return;
 
-			if(nSelCount == 0) return;
+			// LVSIC: one access by index requires O(n) time, thus copy
+			// all to an array (which requires O(1) for each element)
+			int[] v = new int[n];
+			lvsc.CopyTo(v, 0);
 
-			for(int i = 0; i < lvsi.Count; ++i)
-				m_vHistory.Remove(m_vHistory.GetAt((uint)lvsi[nSelCount - i - 1]));
+			for(int i = 0; i < n; ++i)
+				m_vHistory.RemoveAt((uint)v[n - i - 1]);
 
-			UpdateHistoryList();
+			UpdateHistoryList(true);
 			ResizeColumnHeaders();
 		}
 
@@ -1335,10 +1353,9 @@ namespace KeePass.Forms
 			}
 			else { Debug.Assert(false); }
 
-			UpdateEntryStrings(true, false);
+			UpdateEntryStrings(true, false, false);
 			m_vStrings.Remove(lvi.Text);
-			UpdateEntryStrings(false, false);
-			EnableControlsEx();
+			UpdateEntryStrings(false, false, true);
 		}
 
 		private void OnBtnStrMove(object sender, EventArgs e)
@@ -1368,16 +1385,14 @@ namespace KeePass.Forms
 
 		private void OnPwGenOpen(object sender, EventArgs e)
 		{
-			PwGeneratorForm pgf = new PwGeneratorForm();
-
 			byte[] pbCurPassword = m_icgPassword.GetPasswordUtf8();
 			bool bAtLeastOneChar = (pbCurPassword.Length > 0);
 			ProtectedString ps = new ProtectedString(true, pbCurPassword);
 			Array.Clear(pbCurPassword, 0, pbCurPassword.Length);
 			PwProfile opt = PwProfile.DeriveFromPassword(ps);
 
+			PwGeneratorForm pgf = new PwGeneratorForm();
 			pgf.InitEx((bAtLeastOneChar ? opt : null), true, false);
-			// pgf.InitEx(null, true, false);
 
 			if(pgf.ShowDialog() == DialogResult.OK)
 			{
@@ -1397,21 +1412,24 @@ namespace KeePass.Forms
 
 		private void OnProfilesDynamicMenuClick(object sender, DynamicMenuEventArgs e)
 		{
+			string strProfile = (e.Tag as string);
+			if(strProfile == null) { Debug.Assert(false); return; }
+
 			PwProfile pwp = null;
-			if(e.ItemName == DeriveFromPrevious)
+			if(strProfile == DeriveFromPrevious)
 			{
 				byte[] pbCur = m_icgPassword.GetPasswordUtf8();
 				ProtectedString psCur = new ProtectedString(true, pbCur);
 				MemUtil.ZeroByteArray(pbCur);
 				pwp = PwProfile.DeriveFromPassword(psCur);
 			}
-			else if(e.ItemName == AutoGenProfile)
+			else if(strProfile == AutoGenProfile)
 				pwp = Program.Config.PasswordGenerator.AutoGeneratedPasswordsProfile;
 			else
 			{
 				foreach(PwProfile pwgo in PwGeneratorUtil.GetAllProfiles(false))
 				{
-					if(pwgo.Name == e.ItemName)
+					if(pwgo.Name == strProfile)
 					{
 						pwp = pwgo;
 						break;
@@ -1434,8 +1452,10 @@ namespace KeePass.Forms
 		{
 			m_dynGenProfiles.Clear();
 			m_dynGenProfiles.AddSeparator();
-			m_dynGenProfiles.AddItem(DeriveFromPrevious, Properties.Resources.B16x16_CompFile);
-			m_dynGenProfiles.AddItem(AutoGenProfile, Properties.Resources.B16x16_FileNew);
+
+			List<char> lAvailKeys = new List<char>(PwCharSet.MenuAccels);
+			DynAddProfile(DeriveFromPrevious, Properties.Resources.B16x16_CompFile, lAvailKeys);
+			DynAddProfile(AutoGenProfile, Properties.Resources.B16x16_FileNew, lAvailKeys);
 
 			bool bHideBuiltIn = ((Program.Config.UI.UIFlags &
 				(ulong)AceUIFlags.HideBuiltInPwGenPrfInEntryDlg) != 0);
@@ -1455,11 +1475,19 @@ namespace KeePass.Forms
 
 			if(l.Count > 0) m_dynGenProfiles.AddSeparator();
 			foreach(KeyValuePair<string, Image> kvp in l)
-				m_dynGenProfiles.AddItem(kvp.Key, kvp.Value);
+				DynAddProfile(kvp.Key, kvp.Value, lAvailKeys);
 
 			m_ctxPwGen.Show(m_btnGenPw, new Point(0, m_btnGenPw.Height));
 		}
 
+		private void DynAddProfile(string strProfile, Image img, List<char> lAvailKeys)
+		{
+			string strText = StrUtil.EncodeMenuText(strProfile);
+			strText = StrUtil.AddAccelerator(strText, lAvailKeys);
+
+			m_dynGenProfiles.AddItem(strText, img, strProfile);
+		}
+
 		private void OnPickForegroundColor(object sender, EventArgs e)
 		{
 			Color? clr = UIUtil.ShowColorDialog(m_clrForeground);
