@@ -87,6 +87,7 @@ namespace KeePass.Forms
 
 			string strCat = string.Empty;
 			ListViewGroup lvg = null;
+			const uint uMinComp = 2;
 
 			foreach(UpdateComponentInfo uc in m_lInfo)
 			{
@@ -123,12 +124,12 @@ namespace KeePass.Forms
 				else lvi.ImageIndex = 0;
 
 				lvi.SubItems.Add(strStatus);
-				lvi.SubItems.Add(StrUtil.VersionToString(uc.VerInstalled, true));
+				lvi.SubItems.Add(StrUtil.VersionToString(uc.VerInstalled, uMinComp));
 
 				if((uc.Status == UpdateComponentStatus.UpToDate) ||
 					(uc.Status == UpdateComponentStatus.NewVerAvailable) ||
 					(uc.Status == UpdateComponentStatus.PreRelease))
-					lvi.SubItems.Add(StrUtil.VersionToString(uc.VerAvailable, true));
+					lvi.SubItems.Add(StrUtil.VersionToString(uc.VerAvailable, uMinComp));
 				else lvi.SubItems.Add("?");
 
 				if(lvg != null) lvi.Group = lvg;
