@@ -31,8 +31,6 @@ namespace KeePass.UI
 {
 	public sealed class CustomRichTextBoxEx : RichTextBox
 	{
-		// private int m_nClearFormatting = 0;
-
 		private bool m_bSimpleTextOnly = false;
 		[Browsable(false)]
 		[DefaultValue(false)]
@@ -53,15 +51,53 @@ namespace KeePass.UI
 
 		public CustomRichTextBoxEx() : base()
 		{
+			// We cannot use EnableAutoDragDrop, because moving some text
+			// using drag&drop can remove the selected text from the box
+			// (even when read-only is enabled!), which is usually not a
+			// good behavior in the case of KeePass;
+			// reproducible e.g. with LibreOffice Writer, not WordPad
 			// this.EnableAutoDragDrop = true;
+
+			// this.AllowDrop = true;
+
+			// The following is intended to set a default value;
+			// also see OnHandleCreated
+			this.AutoWordSelection = false;
 		}
 
-/* #if DEBUG
-		~CustomRichTextBoxEx()
+		private CriticalSectionEx m_csAutoProps = new CriticalSectionEx();
+		protected override void OnHandleCreated(EventArgs e)
 		{
-			Debug.Assert(m_nClearFormatting == 0);
+			base.OnHandleCreated(e);
+
+			// The following operations should not recreate the handle
+			if(m_csAutoProps.TryEnter())
+			{
+				// Workaround for .NET bug:
+				// AutoWordSelection remembers the value of the property
+				// correctly, but incorrectly sends a message as follows:
+				//   SendMessage(EM_SETOPTIONS, value ? ECOOP_OR : ECOOP_XOR,
+				//       ECO_AUTOWORDSELECTION);
+				// So, when setting AutoWordSelection to false, the control
+				// style is toggled instead of turned off (the internal value
+				// is updated correctly)
+				bool bAutoWord = this.AutoWordSelection; // Internal, no message
+				if(!bAutoWord) // Only 'false' needs workaround
+				{
+					// Ensure control style is on (currently we're in a
+					// random state, as it could be set to false multiple
+					// times; e.g. base.OnHandleCreated does the following:
+					// 'this.AutoWordSelection = this.AutoWordSelection;')
+					this.AutoWordSelection = true;
+
+					// Toggle control style to false
+					this.AutoWordSelection = false;
+				}
+
+				m_csAutoProps.Exit();
+			}
+			else { Debug.Assert(false); }
 		}
-#endif */
 
 		private static bool IsPasteCommand(KeyEventArgs e)
 		{
@@ -149,97 +185,11 @@ namespace KeePass.UI
 			catch(Exception) { Debug.Assert(false); }
 		}
 
-		/* private void ClearFormattingEx()
-		{
-			try
-			{
-				int iSelStart = this.SelectionStart;
-				int iSelLen = this.SelectionLength;
-				string strText = this.Text;
-
-				Clear();
-				this.Text = strText;
-
-				Select(iSelStart, iSelLen);
-				ScrollToCaret();
-			}
-			catch(Exception) { Debug.Assert(false); }
-		}
-
-		private bool DropHasText(DragEventArgs e)
-		{
-			if(e == null) { Debug.Assert(false); return false; }
-
-			IDataObject ido = e.Data;
-			if(ido == null)
-			{
-				Debug.Assert(false);
-				e.Effect = DragDropEffects.None;
-				return false;
-			}
-
-			return (ido.GetDataPresent(DataFormats.Text) ||
-				ido.GetDataPresent(DataFormats.UnicodeText));
-		}
-
-		private void CustomizeDropEffect(DragEventArgs e)
-		{
-			if(e == null) { Debug.Assert(false); return; }
-			if(!DropHasText(e)) e.Effect = DragDropEffects.None;
-		}
-
-		protected override void OnDragEnter(DragEventArgs drgevent)
-		{
-			base.OnDragEnter(drgevent);
-			CustomizeDropEffect(drgevent);
-		}
-
-		protected override void OnDragOver(DragEventArgs drgevent)
-		{
-			base.OnDragOver(drgevent);
-			CustomizeDropEffect(drgevent);
-		}
-
-		protected override void OnDragDrop(DragEventArgs drgevent)
-		{
-			if(drgevent == null) { Debug.Assert(false); return; }
-			if(!DropHasText(drgevent))
-			{
-				drgevent.Effect = DragDropEffects.None;
-				return;
-			}
-
-			base.OnDragDrop(drgevent);
-
-			if(m_bSimpleTextOnly && (drgevent.Effect != DragDropEffects.None))
-			{
-				Debug.Assert(m_nClearFormatting == 0);
-				++m_nClearFormatting;
-			}
-		}
-
-		protected override void OnTextChanged(EventArgs e)
-		{
-			base.OnTextChanged(e);
-
-			if(m_nClearFormatting > 0)
-			{
-				Debug.Assert(m_nClearFormatting == 1);
-				--m_nClearFormatting; // *Before* changing the text again!
-				ClearFormattingEx();
-			}
-		} */
-
-		// protected override void OnMouseDown(MouseEventArgs e)
-		// {
-		//	string strText = this.SelectedText;
-		//	if(!string.IsNullOrEmpty(strText))
-		//		this.DoDragDrop(this.SelectedText, DragDropEffects.Copy |
-		//			DragDropEffects.Scroll);
-		//	else base.OnMouseDown(e);
-		// }
+		// //////////////////////////////////////////////////////////////////
+		// Drag&Drop Source Support
 
 		/* private Rectangle m_rectDragBox = Rectangle.Empty;
+		private bool m_bCurDragSource = false;
 
 		protected override void OnMouseDown(MouseEventArgs e)
 		{
@@ -257,12 +207,13 @@ namespace KeePass.UI
 		protected override void OnMouseUp(MouseEventArgs mevent)
 		{
 			m_rectDragBox = Rectangle.Empty;
+
 			base.OnMouseUp(mevent);
 		}
 
 		protected override void OnMouseMove(MouseEventArgs e)
 		{
-			base.OnMouseMove(e);
+			if(m_bCurDragSource) { base.OnMouseMove(e); return; }
 
 			if(((e.Button & MouseButtons.Left) == MouseButtons.Left) &&
 				!m_rectDragBox.IsEmpty)
@@ -272,19 +223,197 @@ namespace KeePass.UI
 					m_rectDragBox = Rectangle.Empty;
 
 					string strText = this.SelectedText;
-					int iSelStart = this.SelectionStart;
-					int iSelLen = this.SelectionLength;
+					// int iSelStart = this.SelectionStart;
+					// int iSelLen = this.SelectionLength;
 
 					if(!string.IsNullOrEmpty(strText))
 					{
-						DoDragDrop(strText, DragDropEffects.Copy |
-							DragDropEffects.Scroll);
-						Select(iSelStart, iSelLen);
+						m_bCurDragSource = true;
+						DoDragDrop(strText, (DragDropEffects.Move |
+							DragDropEffects.Copy | DragDropEffects.Scroll));
+						m_bCurDragSource = false;
+
+						// Select(iSelStart, iSelLen);
+						return;
 					}
 					else { Debug.Assert(false); }
 				}
 			}
-			else m_rectDragBox = Rectangle.Empty;
+
+			base.OnMouseMove(e);
+		} */
+
+		// //////////////////////////////////////////////////////////////////
+		// Drag&Drop Target Support
+
+		/* private static bool ObjectHasText(DragEventArgs e)
+		{
+			if(e == null) { Debug.Assert(false); return false; }
+
+			IDataObject ido = e.Data;
+			if(ido == null)
+			{
+				Debug.Assert(false);
+				return false;
+			}
+
+			return (ido.GetDataPresent(DataFormats.Text) ||
+				ido.GetDataPresent(DataFormats.UnicodeText));
+		}
+
+		private bool IsDropAcceptable(DragEventArgs e)
+		{
+			if(e == null) { Debug.Assert(false); return false; }
+
+			if(this.ReadOnly) return false;
+
+			// Currently, in-control drag&drop is unsupported
+			if(m_bCurDragSource) return false;
+
+			return ObjectHasText(e);
+		}
+
+		private bool CustomizeDropEffect(DragEventArgs e)
+		{
+			if(e == null) { Debug.Assert(false); return false; }
+
+			if(!IsDropAcceptable(e))
+			{
+				e.Effect = DragDropEffects.None;
+				return true;
+			}
+
+			return false;
+		}
+
+		protected override void OnDragEnter(DragEventArgs drgevent)
+		{
+			base.OnDragEnter(drgevent);
+			CustomizeDropEffect(drgevent);
+		}
+
+		protected override void OnDragOver(DragEventArgs drgevent)
+		{
+			base.OnDragOver(drgevent);
+			CustomizeDropEffect(drgevent);
+		}
+
+		protected override void OnDragDrop(DragEventArgs drgevent)
+		{
+			if(drgevent == null) { Debug.Assert(false); return; }
+			if(!IsDropAcceptable(drgevent))
+			{
+				drgevent.Effect = DragDropEffects.None;
+				return;
+			}
+
+			if(m_bSimpleTextOnly)
+			{
+				IDataObject d = drgevent.Data;
+				string str = null;
+				if(d.GetDataPresent(DataFormats.UnicodeText))
+					str = (d.GetData(DataFormats.UnicodeText) as string);
+				if((str == null) && d.GetDataPresent(DataFormats.Text))
+					str = (d.GetData(DataFormats.Text) as string);
+				if(str == null)
+				{
+					Debug.Assert(false);
+					drgevent.Effect = DragDropEffects.None;
+					return;
+				}
+
+				Point pt = new Point(drgevent.X, drgevent.Y);
+				pt = PointToClient(pt);
+
+				drgevent.Effect = DragDropEffects.None;
+
+				int pIns = GetCharIndexFromPosition(pt);
+				InsertTextAt(str, pIns);
+			}
+			else base.OnDragDrop(drgevent);
+		}
+
+		private void InsertTextAt(string str, int pIns)
+		{
+			if(string.IsNullOrEmpty(str)) return;
+
+			string strText = this.Text;
+
+			if((pIns < 0) || (pIns > strText.Length)) { Debug.Assert(false); return; }
+
+			this.Text = strText.Insert(pIns, str);
+
+			Select(pIns + str.Length, 0); // Data was inserted, not moved
+			ScrollToCaret();
+		} */
+
+		// //////////////////////////////////////////////////////////////////
+		// Simple Selection Support
+
+		// The following selection code is not required, because
+		// AutoWordSelection can be used with a workaround
+
+		// private int? m_oiMouseSelStart = null;
+
+		/* protected override void OnMouseDown(MouseEventArgs e)
+		{
+			base.OnMouseDown(e);
+
+			if(((e.Button & MouseButtons.Left) != MouseButtons.None) &&
+				(this.SelectionLength == 0))
+				m_oiMouseSelStart = this.SelectionStart;
+		}
+
+		protected override void OnMouseMove(MouseEventArgs e)
+		{
+			if(UpdateSelectionEx(e)) return;
+
+			base.OnMouseMove(e);
+		}
+
+		protected override void OnMouseUp(MouseEventArgs mevent)
+		{
+			if((mevent.Button & MouseButtons.Left) != MouseButtons.None)
+				m_oiMouseSelStart = null;
+
+			base.OnMouseUp(mevent);
+		}
+
+		private bool UpdateSelectionEx(MouseEventArgs e)
+		{
+			if(!m_oiMouseSelStart.HasValue) return false;
+			if((e.Button & MouseButtons.Left) == MouseButtons.None) return false;
+
+			try
+			{
+				int iSelS = m_oiMouseSelStart.Value;
+				int iSelE = Math.Max(GetCharIndexFromPosition(e.Location), 0);
+
+				if(iSelE == (this.TextLength - 1))
+				{
+					Font f = this.SelectionFont;
+					if(f == null)
+					{
+						Select(iSelE, 1);
+						f = (this.SelectionFont ?? this.Font);
+					}
+
+					// Measuring a single character is imprecise (padding, ...)
+					Size szLastChar = TextRenderer.MeasureText(new string(
+						this.Text[iSelE], 20), f);
+					int wLastChar = szLastChar.Width / 20;
+
+					Point ptLastChar = GetPositionFromCharIndex(iSelE);
+
+					if(e.X >= (ptLastChar.X + (wLastChar / 2))) ++iSelE;
+				}
+
+				if(iSelS <= iSelE) Select(iSelS, iSelE - iSelS);
+				else Select(iSelE, iSelS - iSelE);
+			}
+			catch(Exception) { Debug.Assert(false); return false; }
+
+			return true;
 		} */
 	}
 }
