@@ -33,6 +33,8 @@ using KeePassLib;
 using KeePassLib.Cryptography;
 using KeePassLib.Utility;
 
+// using KpLibNativeMethods = KeePassLib.Native.NativeMethods;
+
 namespace KeePass.UI
 {
 	public delegate Form UIFormConstructor(object objParam);
@@ -206,6 +208,7 @@ namespace KeePass.UI
 		private void SecureDialogThread(object oParam)
 		{
 			BackgroundForm formBack = null;
+			// bool bLangBar = false;
 
 			SecureThreadInfo stp = (oParam as SecureThreadInfo);
 			if(stp == null) { Debug.Assert(false); return; }
@@ -250,6 +253,8 @@ namespace KeePass.UI
 				if(Program.Config.UI.SecureDesktopPlaySound)
 					UIUtil.PlayUacSound();
 
+				// bLangBar = ShowLangBar(true);
+
 				lock(stp) { stp.State = SecureThreadState.ShowingDialog; }
 				stp.DialogResult = f.ShowDialog(formBack);
 				stp.ResultObject = m_fnResultBuilder(f);
@@ -259,6 +264,8 @@ namespace KeePass.UI
 			catch(Exception) { Debug.Assert(false); }
 			finally
 			{
+				// if(bLangBar) ShowLangBar(false);
+
 				if(formBack != null)
 				{
 					try
@@ -273,6 +280,19 @@ namespace KeePass.UI
 			}
 		}
 
+		/* private static bool ShowLangBar(bool bShow)
+		{
+			try
+			{
+				return KpLibNativeMethods.TfShowLangBar(bShow ?
+					KpLibNativeMethods.TF_SFT_SHOWNORMAL :
+					KpLibNativeMethods.TF_SFT_HIDDEN);
+			}
+			catch(Exception) { }
+
+			return false;
+		} */
+
 		private static void HandleUnexpectedDesktopSwitch(IntPtr pOrgDesktop,
 			IntPtr pNewDesktop, SecureThreadInfo stp)
 		{
